package com.gree.mall.manager.logic.material.base;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.material.base.WorkerGoodsBean;
import com.gree.mall.manager.bean.material.base.WorkerGoodsVO;
import com.gree.mall.manager.commonmapper.MaterialMapper;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.WorkerGoods;
import com.gree.mall.manager.plus.entity.WorkerGoodsRela;
import com.gree.mall.manager.plus.service.WorkerGoodsRelaService;
import com.gree.mall.manager.plus.service.WorkerGoodsService;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class WorkerGoodsLogic {

    private final CommonLogic commonLogic;
    private final MaterialMapper materialMapper;
    private final WorkerGoodsService workerGoodsService;
    private final WorkerGoodsRelaService workerGoodsRelaService;

    public IPage<WorkerGoodsVO> page(ZfireParamBean zfireParamBean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        zfireParamBean.setCompanyWechatId(adminUser.getCompanyWechatId());
        FieldUtils.supplyParam(zfireParamBean, WorkerGoodsVO.class);
        return materialMapper.workGoodsPage(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
    }

    public WorkerGoodsBean detail(String id) {
        WorkerGoods workerGoods = workerGoodsService.getById(id);
        WorkerGoodsBean bean = new WorkerGoodsBean();
        BeanUtils.copyProperties(bean, workerGoods);
        List<WorkerGoodsRela> goodsRelas = workerGoodsRelaService.lambdaQuery()
                .eq(WorkerGoodsRela::getWorkerGoodsId, id)
                .list();
        bean.setItems(goodsRelas);
        return bean;
    }

    @Transactional(rollbackFor = Exception.class)
    public void add(WorkerGoodsBean bean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        bean.setGoodsId(null);
        bean.setCompanyWechatId(adminUser.getCompanyWechatId());

        WorkerGoods workerGoods = new WorkerGoods();
        BeanUtils.copyProperties(bean, workerGoods);

        workerGoods.insert();

        if (CollectionUtil.isNotEmpty(bean.getItems())) {
            for (WorkerGoodsRela item : bean.getItems()) {
                item.setCompanyWechatId(workerGoods.getCompanyWechatId())
                    .setWorkerGoodsId(workerGoods.getGoodsId());
            }
            workerGoodsRelaService.saveBatch(bean.getItems());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void edit(WorkerGoodsBean bean) {
        AdminUserCom adminUser = commonLogic.getAdminUser();
        bean.setCompanyWechatId(adminUser.getCompanyWechatId());

        WorkerGoods workerGoods = new WorkerGoods();
        BeanUtils.copyProperties(bean, workerGoods);

        workerGoods.updateById();

        if (CollectionUtil.isNotEmpty(bean.getItems())) {
            workerGoodsRelaService.lambdaUpdate()
                    .eq(WorkerGoodsRela::getCompanyWechatId, workerGoods.getCompanyWechatId())
                    .eq(WorkerGoodsRela::getWorkerGoodsId, workerGoods.getGoodsId())
                    .remove();
            for (WorkerGoodsRela item : bean.getItems()) {
                item.setCompanyWechatId(workerGoods.getCompanyWechatId())
                        .setWorkerGoodsId(workerGoods.getGoodsId());
            }
            workerGoodsRelaService.saveBatch(bean.getItems());
        }
    }
}
